/*
 * Copyright (c) 1987, 1988, 1989, 1990, 1991 Stanford University
 * Copyright (c) 1991 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * Stanford and Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Stanford and Silicon Graphics.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 *
 * IN NO EVENT SHALL STANFORD OR SILICON GRAPHICS BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

/*
 * Coordinates for graphics allocation and rendering.
 */

#ifndef iv_coord_h
#define iv_coord_h

#include <InterViews/enter-scope.h>

typedef float _lib_iv(Coord);
typedef float _lib_iv(FloatCoord);
typedef int _lib_iv(IntCoord);
typedef int _lib_iv(PixelCoord);

/*
 * Should be a static const float, but some compilers don't
 * do a very good job on that.
 */
#define fil 10e6

/*
 * Old definitions
 */
typedef int _lib_iv2_6(Coord);

#ifndef PDP_PLUS_PLUS
extern double inch, inches, cm, mm, point, points;
static const int pixels = 1;
#define ivinch inch
#define ivinches inches
#define ivcm cm
#define ivmm mm
#define ivpoint point
#define ivpoints points
#define ivpixels pixels
#else
extern double ivinch, ivinches, ivcm, ivmm, ivpoint, ivpoints;
static const int ivpixels = 1;
#endif

#endif
