/*
 * Copyright (c) 1990, 1991 Stanford University
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Stanford not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  Stanford makes no representations about
 * the suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * STANFORD DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL STANFORD BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * Implemementation of Data subclasses.
 */

#include <Unidraw/Commands/datas.h>
#include <Unidraw/Graphic/graphic.h>

#include <IV-2_6/_enter.h>

/*****************************************************************************/

VoidData::VoidData (void* v) {
    _void = v;
}

/*****************************************************************************/

ColorData::ColorData (PSColor* fg, PSColor* bg) {
    _fg = fg;
    _bg = bg;
}

/*****************************************************************************/

MoveData::MoveData (float dx, float dy) {
    _dx = dx;
    _dy = dy;
}

/*****************************************************************************/

GSData::GSData (Graphic* g) { _gs = new FullGraphic(g); }
GSData::~GSData () { delete _gs; }

/*****************************************************************************/

MobilityData::MobilityData (Mobility m, Graphic* g) { 
    _mobility = m;
    _gs = new FullGraphic(g);
}

MobilityData::~MobilityData () {
    delete _gs;
}

/*****************************************************************************/

UngroupData::UngroupData (GraphicComp* p, Graphic* g) { 
    _parent = p;
    _gs = new FullGraphic(g);
}

UngroupData::~UngroupData () {
    delete _gs;
}


