/*
 * Copyright (c) 1987, 1988, 1989, 1990, 1991 Stanford University
 * Copyright (c) 1991 Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * Stanford and Silicon Graphics may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of Stanford and Silicon Graphics.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 *
 * IN NO EVENT SHALL STANFORD OR SILICON GRAPHICS BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

/*
 * Target - hit sensitivity control
 */

#include <InterViews/hit.h>
#include <InterViews/target.h>

Target::Target(Glyph* body, TargetSensitivity sensitivity) : MonoGlyph(body) {
    sensitivity_ = sensitivity;
}

Target::~Target() { }

void Target::pick(Canvas* c, const Allocation& a, int depth, Hit& h) {
    Coord x, left, right;
    switch (sensitivity_) {
    case TargetBodyHit:
	MonoGlyph::pick(c, a, depth, h);
	break;
    case TargetNeverHit:
	break;
    case TargetAlwaysHit:
	h.target(depth, this, 0);
	break;
    case TargetPrimitiveHit:
        if (h.right() >= a.left() && h.left() < a.right() &&
	    h.top() >= a.bottom() && h.bottom() < a.top()
	) {
            h.target(depth, this, 0);
        }
	break;
    case TargetCharacterHit:
	/* call to pick is dubious, but needed for now by doc */
	MonoGlyph::pick(c, a, depth, h);
	x = h.left();
        left = a.left();
        right = a.right();
        if (h.right() >= left && x < right) {
            h.target(depth, this, (x > ((left + right) * 0.5)) ? 1 : 0);
        }
	break;
    }
}
