#undef ClassId
#undef Orientation
#undef Zooming
#undef AcknowledgeDialog
#undef AlignCmd
#undef AlignToGridCmd
#undef BackCmd
#undef BasicDialog
#undef BoxObj
#undef BrushCmd
#undef BrushVar
#undef BrushVarView
#undef CCnxn
#undef CCnxn_HashTable
#undef CGlue
#undef CNet
#undef CSolver
#undef CSolverInfo
#undef CSolverState
#undef Catalog
#undef CatalogChooser
#undef CenterCmd
#undef Clipboard
#undef CloseEditorCmd
#undef ClosedBSpline
#undef ClosedSplineComp
#undef ClosedSplineView
#undef ColorCmd
#undef ColorData
#undef ColorVar
#undef Command
#undef CommandControl
#undef CommandInteractor
#undef CompNameVar
#undef CompNameVarView
#undef Component
#undef ComponentView
#undef ConfirmDialog
#undef ConnInfo
#undef ConnectCmd
#undef ConnectManip
#undef ConnectTool
#undef Connector
#undef ConnectorView
#undef ControlInfo
#undef CopyCmd
#undef Creator
#undef CutCmd
#undef Damage
#undef Data
#undef DataCache
#undef DeleteCmd
#undef DragManip
#undef DupCmd
#undef Editor
#undef EditorImpl
#undef EditorInfo
#undef Ellipse
#undef EllipseComp
#undef EllipseView
#undef Extent
#undef ExternView
#undef F_Circle
#undef F_ClosedBSpline
#undef F_Ellipse
#undef F_OpenBSpline
#undef F_Polygon
#undef F_Rect
#undef FileNameVarView
#undef FillPolygonObj
#undef FixedPin
#undef FontCmd
#undef FontVar
#undef FontVarView
#undef FrontCmd
#undef FullGraphic
#undef GSData
#undef GVU_HashTable
#undef GVUpdater
#undef Graphic
#undef GraphicBlock
#undef GraphicComp
#undef GraphicCompTool
#undef GraphicComps
#undef GraphicView
#undef GraphicViews
#undef GravityCmd
#undef GravityVar
#undef GravityVarView
#undef Grid
#undef GridCmd
#undef GridDialog
#undef GridSpacingCmd
#undef GroupCmd
#undef HPanelControl
#undef HSlotComp
#undef HSlotView
#undef HistoryMap
#undef ImportCmd
#undef InorderView
#undef Iterator
#undef KeyMap
#undef Line
#undef LineComp
#undef LineObj
#undef LineView
#undef LinkComp
#undef LinkView
#undef MacroCmd
#undef MagnifVar
#undef MagnifVarView
#undef MagnifyTool
#undef ManagedWindow
#undef ManipGroup
#undef Manipulator
#undef MatchEditor
#undef MobilityCmd
#undef MobilityData
#undef ModifStatusVar
#undef ModifStatusVarView
#undef MoveCmd
#undef MoveData
#undef MoveTool
#undef MultiLine
#undef MultiLineComp
#undef MultiLineObj
#undef MultiLineView
#undef NOPCmd
#undef NameMap
#undef NameVar
#undef NameVarView
#undef NewCompCmd
#undef NormSizeCmd
#undef ObjectMap
#undef ObjectMapElem
#undef OpenBSpline
#undef OrientationCmd
#undef PSBrush
#undef PSClosedSpline
#undef PSColor
#undef PSEllipse
#undef PSLine
#undef PSLink
#undef PSMultiLine
#undef PSPad
#undef PSPattern
#undef PSPin
#undef PSPolygon
#undef PSRaster
#undef PSRect
#undef PSSlot
#undef PSSpline
#undef PSStencil
#undef PSText
#undef PSVertices
#undef PadComp
#undef PadGraphic
#undef PadView
#undef Page
#undef PanelControl
#undef PanelInteractor
#undef PasteCmd
#undef Path
#undef PatternCmd
#undef PatternVar
#undef PatternVarView
#undef Picture
#undef PinComp
#undef PinGraphic
#undef PinView
#undef Point
#undef PointObj
#undef Polygon
#undef PolygonComp
#undef PolygonView
#undef PostScriptView
#undef PostScriptViews
#undef PostorderView
#undef PreorderView
#undef PrintBS
#undef PrintCmd
#undef PrintDialog
#undef QuitCmd
#undef RasterComp
#undef RasterRect
#undef RasterView
#undef RectComp
#undef RectView
#undef RedToFitCmd
#undef RedoCmd
#undef ReplaceCmd
#undef ReshapeTool
#undef RevertCmd
#undef RotateCmd
#undef RotateTool
#undef SFH_ClosedBSpline
#undef SFH_OpenBSpline
#undef SF_Circle
#undef SF_ClosedBSpline
#undef SF_Ellipse
#undef SF_MultiLine
#undef SF_OpenBSpline
#undef SF_Polygon
#undef SF_Rect
#undef S_Circle
#undef S_ClosedBSpline
#undef S_Ellipse
#undef S_MultiLine
#undef S_OpenBSpline
#undef S_Polygon
#undef S_Rect
#undef SaveCompAsCmd
#undef SaveCompCmd
#undef ScaleCmd
#undef ScaleTool
#undef SelectTool
#undef SlctAllCmd
#undef SlidingPin
#undef SlotComp
#undef SlotGraphic
#undef SlotView
#undef SplineComp
#undef SplineView
#undef State
#undef StateVar
#undef StateVarView
#undef StencilComp
#undef StencilView
#undef StretchTool
#undef TF_2Port
#undef TF_Direct
#undef TextBuffer
#undef TextComp
#undef TextGraphic
#undef TextInteractor
#undef TextManip
#undef TextView
#undef Tool
#undef TransferFunct
#undef UArray
#undef UChooser
#undef UControl
#undef UControlInteractor
#undef UHashElem
#undef UHashTable
#undef ULabel
#undef UList
#undef UMap
#undef UMapElem
#undef UPage
#undef UStencil
#undef UndoCmd
#undef UngroupCmd
#undef Unidraw
#undef VPanelControl
#undef VSlotComp
#undef VSlotView
#undef VertexManip
#undef Vertices
#undef VerticesComp
#undef VerticesView
#undef ViewCompCmd
#undef Viewer
#undef ViewerView
#undef VoidData
